@@ -9,8 +9,6 @@
 #include <KColorScheme>
 #include <KColorUtils>
 #include <KConfigGroup>
-#include <KIconLoader>
-#include <QDBusConnection>
 #include <QDebug>
 #include <QGuiApplication>
 #include <QPalette>
@@ -18,16 +16,13 @@
 #include <QQmlEngine>
 #include <QQuickRenderControl>
 #include <QQuickWindow>
+#include <QScopeGuard>
 
-class IconLoaderSingleton
-{
-public:
-    IconLoaderSingleton() = default;
+#ifndef Q_OS_ANDROID
+#include <QDBusConnection>
 
-    KIconLoader self;
-};
-
-Q_GLOBAL_STATIC(IconLoaderSingleton, privateIconLoaderSelf)
+#include <KIconLoader>
+#endif
 
 class StyleSingleton : public QObject
 {
@@ -46,6 +41,7 @@ public:
     {
         connect(qGuiApp, &QGuiApplication::paletteChanged, this, &StyleSingleton::refresh);
 
+#ifndef Q_OS_ANDROID
         // Use DBus in order to listen for kdeglobals changes directly, as the
         // QApplication doesn't expose the font variants we're looking for,
         // namely smallFont.
@@ -55,6 +51,7 @@ public:
                                               QStringLiteral("notifyChange"),
                                               this,
                                               SLOT(notifyWatchersConfigurationChange()));
+#endif
 
         connect(qGuiApp, &QGuiApplication::fontDatabaseChanged, this, &StyleSingleton::notifyWatchersConfigurationChange);
 
@@ -199,7 +196,10 @@ PlasmaDesktopTheme::PlasmaDesktopTheme(QObject *parent)
     // TODO: MOVE THIS SOMEWHERE ELSE
     m_lowPowerHardware = QByteArrayList{"1", "true"}.contains(qgetenv("KIRIGAMI_LOWPOWER_HARDWARE").toLower());
 
+    // We don't use KIconLoader on Android so we don't support recoloring there
+#ifndef Q_OS_ANDROID
     setSupportsIconColoring(true);
+#endif
 
     auto parentItem = qobject_cast<QQuickItem *>(parent);
     if (parentItem) {
@@ -266,6 +266,7 @@ void PlasmaDesktopTheme::syncFont()
 
 QIcon PlasmaDesktopTheme::iconFromTheme(const QString &name, const QColor &customColor)
 {
+#ifndef Q_OS_ANDROID
     QPalette pal = palette();
     if (customColor != Qt::transparent) {
         for (auto state : {QPalette::Active, QPalette::Inactive, QPalette::Disabled}) {
@@ -273,9 +274,23 @@ QIcon PlasmaDesktopTheme::iconFromTheme(const QString &name, const QColor &custo
         }
     }
 
-    privateIconLoaderSelf->self.setCustomPalette(pal);
+    bool hadPalette = KIconLoader::global()->hasCustomPalette();
+    QPalette olderPalette = KIconLoader::global()->customPalette();
 
-    return KDE::icon(name, &privateIconLoaderSelf->self);
+    auto cleanup = qScopeGuard([&] {
+        if (hadPalette) {
+            KIconLoader::global()->setCustomPalette(olderPalette);
+        } else {
+            KIconLoader::global()->resetPalette();
+        }
+    });
+
+    KIconLoader::global()->setCustomPalette(pal);
+    return KDE::icon(name, KIconLoader::global());
+#else
+    // On Android we don't want to use the KIconThemes-based loader since that appears to be broken
+    return QIcon::fromTheme(name);
+#endif
 }
 
 void PlasmaDesktopTheme::syncColors()
